/*jslint bitwise: true, continue: true, es5: true, plusplus: true, undef: true, sloppy: true, stupid: false, vars: true, white: true, passfail: false, indent: 2, maxerr: 50 */

// hueControl digitalSTROM server add-on
// for controlling Philips Hue LED lamps
// (c) 2012 by plan44.ch
//
// Configuration (management of the config property)



// compare versions
function isPropertyTreeNotCurrent(currentVersionString, compareVersionString) {
  var curVersion = currentVersionString.split('.');
  var comVersion = compareVersionString.split('.');
  var i;
  for (i = 0; i < curVersion.length; i++) {
    if (
      (i < comVersion.length) &&
      (parseInt(curVersion[i], 10) < parseInt(comVersion[i], 10))
    ) {
      return true;
    }
  }
  return false;
}


function migrateConfig(currentVersion, newVersion) {
  lInfo.logln("Requested migration from version " + currentVersion + " to " + newVersion);
  //%%% for now: never possible to upgrade
  return false;
}


// http://192.168.59.107/api/1cb6220dda14fc736331ec93917b7e3/lights/3/state --request PUT --data '{ "on":true}'

// prepare config for use - create default entries where needed etc.
function prepareConfig()
{
  lInfo.logln("preparing config");
  if (hueBridgeRegisterApp()) {
    // successfully registered with the hue bridge
    // - create lights config container
    var lightsPath = 'lights';
    if (!Property.getNode(lightsPath)) {
      lInfo.logln("creating default light scenes");
      var nls = new Property(lightsPath); // create lights
      nls.setFlag('ARCHIVE', true);
    }
    // make sure we have a config for every light the bridge knows
    // - get list of hue lights
    var lights = hueLights();
    // - request config for each of them, which creates default config when none already exists
    for (var li=0; li<lights.length; li++) {
      var light = lights[li];
      var lightID = light.lightID;
      var lightName = light.name;
      var lightConfig = lightConfigProperty(lightID,lightName);
    }    
  }
}



// Called on startup of the dss, must load the config and possibly migrate it from previous versions
function startup() {
  // config version
  var myConfigVersion = '0.1.0';
  // Get current config
  Property.load();
  var versionNode = Property.getNode('version');
  lInfo.logln("startup hueControl");
  // check current config, if any
  if (versionNode) {
    var currentVersion = versionNode.getValue();
    if (isPropertyTreeNotCurrent(currentVersion, myConfigVersion)) {
      // needs update
      if (!migrateConfig(currentVersion, myConfigVersion)) {
        // cannot update
        lWarn.logln("Cannot migrate config to version " + myConfigVersion);
      }
    }
  }
  // prepare config, create defaults where needed
  prepareConfig();
  // Now assume config to be up-to-date
  Property.setProperty('version', myConfigVersion);
  Property.setFlag('version', 'ARCHIVE', true);
  Property.store();
  lInfo.logln("hueControl running (configVersion " + myConfigVersion + ")");
}



// save hue bridge URL
function saveBridgeURL(aBridgeURL)
{
  // save the URL
  var bridgeURLPath = 'bridgeURL';
  Property.setProperty(bridgeURLPath, aBridgeURL);
  Property.setFlag(bridgeURLPath, 'ARCHIVE', true);
  // store
  Property.store();
}



// update light names from actual hue lights
function updateLightNames()
{
  var lights = hueLights();
  for (li=0; li<lights.length; li++) {
    var light = lights[li];
    var lightPath = 'lights/light' + String(light.lightID);
    var lightNamePath = lightPath + '/name';
    Property.setProperty(lightNamePath, light.name);
  }
  Property.store();
  lInfo.logln("- updated light names: " + JSON.stringify(lights));  
}




// save single light parameters
function saveLight(aValues)
{
  var lightID = parseInt(aValues.lightID);
  var lightPath = 'lights/light' + String(lightID);
  // update name and zoneID
  // - name
  var lightNamePath = lightPath + '/name';
  var lightName = unescape(aValues.name);
  Property.setProperty(lightNamePath, lightName);
  // - also try to update name in hue
  hueCommand('/lights/' + String(lightID), { "name" : lightName });
  // - zoneID
  var zoneIDPath = lightPath + '/zoneID';      
  Property.setProperty(zoneIDPath, parseInt(aValues.zoneID));
  // - Don't Care
  for (var sKey in aValues) {
	lInfo.logln("parameter " + sKey);
	if (sKey.substr(0,8)=='dontCare') {
		var sceneNumber=sKey.substr(8);
		Property.setProperty( lightPath +"/scene" + sceneNumber + "/dontCare",aValues[sKey]);
		Property.setFlag(lightPath +"/scene" + sceneNumber + "/dontCare", 'ARCHIVE', true);
	}
	
  }	
//  var areas = JSON.parse(unescape(aValues.areas));
//  var areasPath = lightPath + '/areas';
//  Property.setProperty(areasPath, JSON.stringify(areas));
  // store
  Property.store();
  lInfo.logln("- saved light params: " + JSON.stringify(aValues));  
}



function performLightSceneOnLight(aLightScene, aLightID)
{
  lInfo.logln("performLightScene " + String(aLightScene) + " OnLight " + String(aLightID));
  var lightConfig = Property.getNode('lights/light'+String(aLightID));
  var stateTextProp = lightConfig.getChild('scene' + String(aLightScene) + '/state');
  var stateText = stateTextProp.getValue();
  lInfo.logln("- state = " + stateText);
  if (stateText) {
    var state = JSON.parse(stateText);
    if (!applyStateToLight(state,aLightID)) {
      lWarn.logln("Can't set state for hue lightID " + String(aLightID));
    }
  }  
}


function saveLightSceneForLight(aLightScene, aLightID)
{
  lInfo.logln("saveLightScene " + String(aLightScene) + " ForLight " + String(aLightID));
  var lightConfig = Property.getNode('lights/light'+String(aLightID));
  var stateTextProp = lightConfig.getChild('scene' + String(aLightScene) + '/state');
  // get current state for light
  var state = getLightState(aLightID);
  if (state===false) {
    lWarn.logln("Can't get state for hue lightID " + String(aLightID));
  }
  else {
    stateTextProp.setValue(JSON.stringify(state));
    stateTextProp.setFlag('ARCHIVE', true);
    Property.store();
  }
}


// called from config UI to save data and perform direct parameter changes on lamps
function handleConfigAction(aAction,aValues)
{
  var values = JSON.parse(unescape(aValues));
  lInfo.logln("received action: " + aAction + " with Values = " + JSON.stringify(values));
  switch (aAction) {
    case 'save':
      saveLight(values);
      var evt = new Event("plan44-addon-huecontrol-config-saved", {});
      evt.raise();
      break;
    case 'lightSceneLoad':
      performLightSceneOnLight(values.lightScene, values.lightID);
      break;
    case 'lightSceneSave':
      saveLightSceneForLight(values.lightScene, values.lightID);
      break;
    case 'updateNames':
      updateLightNames();
      var evt = new Event("plan44-addon-huecontrol-config-saved", {});
      evt.raise();
      break;
    case 'saveBridgeURL':
      saveBridgeURL(values.url);
      // invalidate the token
      var bridgeApiTokenPath = 'bridgeApiToken';
      Property.setProperty(bridgeApiTokenPath, '');
      Property.setFlag(bridgeApiTokenPath, 'ARCHIVE', true);
      // try re-registering
      if (hueBridgeRegisterApp()) {
        // make sure we have config
        prepareConfig();
        // update light names
        updateLightNames();
      }
      // possibly new config - have UI reload data
      lInfo.logln("- done saving new URL");
      var evt = new Event("plan44-addon-huecontrol-config-saved", {});
      evt.raise();
      break;
  }
}




// Event dispatcher

//lInfo.logln("event: " + raisedEvent.name);
if (raisedEvent.name == 'running') {
  // startup of the dss
  // - initialize the add-on
  startup();  
}
else if (raisedEvent.name == 'plan44-addon-huecontrol-config') {
  // request from config UI
  handleConfigAction(raisedEvent.parameter.action, raisedEvent.parameter.values);
}

