function lightSceneLoadButton(aLightScene,aApp)
{
  var button = Ext.create('Ext.Button', {
    text : _('Apply to hue Light'),
    isValid : function() { return true; },
    getValue : function() { return false; },
    handler : function() {
      var lightID = aApp.theGrid.curSelectedEvent.get('lightID');
      var values = { lightScene:aLightScene, lightID:lightID };
      dss.raiseEvent('plan44-addon-huecontrol-config', {
        action : 'lightSceneLoad',
        values : Ext.JSON.encode(values)
      });
    }
  });
  return button;
}


function lightSceneSaveButton(aLightScene,aApp)
{
  var button = Ext.create('Ext.Button', {
    text : _('Save current hue light state to scene'),
    isValid : function() { return true; },
    getValue : function() { return false; },
    handler : function() {
      var lightID = aApp.theGrid.curSelectedEvent.get('lightID');
      var values = { lightScene:aLightScene, lightID:lightID };
      dss.raiseEvent('plan44-addon-huecontrol-config', {
        action : 'lightSceneSave',
        values : Ext.JSON.encode(values)
      });
    }
  });
  return button;
}

function lightSceneDontCare(aLightScene,aApp)
{
  var checkbox = Ext.create('Ext.form.field.Checkbox', {
    layout: 'hbox',
    labelWidth : 20,
    boxLabel : _('Retain output'),
    id : 'area'+aLightScene,
    padding:'1 0 0 10'
  });
  return checkbox;
}


function lightSceneButtons(aLightScene,aApp)
{
  var items = [ lightSceneLoadButton(aLightScene,aApp)];
  var oCheckbox=lightSceneDontCare(aLightScene,aApp)

  var label = '?';
  switch (aLightScene) {
    case  0: label = _('Off'); break;
    case 65: label = _('Panic'); break; 
    case 71: label = _('Present'); break;
    case 72: label = _('Absent'); break;
    case 73: label = _('Bell'); break;
    case 74: label = _('Alarm'); break;
    case 76: label = _('Fire'); break;
    default: label = _('Scene')+' '+String(aLightScene); break;
  }

  items.push(lightSceneSaveButton(aLightScene,aApp));

  items.push(oCheckbox);

  var panel=Ext.create("Ext.panel.Panel", {
        layout:'hbox',
       items:items,
        width:600,
        border:false
        });

  var field = Ext.create('Ext.form.FieldContainer', {
    fieldLabel : label,
    isValid : function() { return true; },
    getValue : function() { 
    	return oCheckbox.getValue(); 
    },
    setValue : function(fValue) {
    	oCheckbox.setValue(fValue);
    },
    items : panel
  });
  return field;
}



function createNeededClasses() {

  // @class huecontrol.bridgeParamEdit
  Ext.define('DSS.addon.huecontrol.bridgeParamEdit', {
  
    extend : 'Ext.window.Window',
  
    buttonSave : null,
    buttonCancel : null,
    bridgeURLTextfield : null,
    bridgeStatusDisplayfield : null,
    
    constructor : function(config) {
      config.title = _('hue bridge configuration');
      config.closeAction = 'hide';
      config.modal = true; 
      config.height = 142;
      config.width = 500;
      config.layout = 'fit';
      config.appName = '';
      config.resizable=false;
      this.mixins.observable.constructor.call(config);
    
      var me = this;
    
      me.buttonCancel = Ext.create('Ext.Button', {
        text : _('cancel'),
        handler : function() {
          me.fireEvent('cancel');
          me.hide();
        }
      });
    
      me.buttonSave = Ext.create('Ext.Button', {
        text : _('save'),
        handler : function() {
          me.fireEvent('save', me.getURLValue());
          me.hide();
        }
      });
    
      me.addEvents({
        cancel : true,
        save : true
      });
    
  
      me.bridgeURLTextfield = Ext.create('Ext.form.field.Text', {
        name : 'bridgeURL',
        width : 450,
        labelWidth : 200,
        fieldLabel : _('hue bridge URL (usually like: http://192.168.x.x)'),
        allowBlank : false
      });
  
      me.bridgeStatusDisplayfield = Ext.create('Ext.form.field.Display', {
        name : 'bridgeStatus',
        width : 450,
        labelWidth : 200,
        fieldLabel : _('bridge status'),
        value : 'unknown'
      });
      
      Ext.apply(me, {
        items : [ {
          xtype : 'panel',
          items : [me.bridgeURLTextfield, me.bridgeStatusDisplayfield],
          bodyPadding : 5,
          border : 0,
          buttons : [ me.buttonSave , me.buttonCancel ]
        } ],
      }); // Ext.apply
    
      this.callParent(arguments);
    }, // constructor
    
    getURLValue : function() {
      return this.bridgeURLTextfield.value;
    },
  
  
    setURLValue : function(aValue) {
      this.bridgeURLTextfield.setValue(aValue);
    },
    
  
    setStateText : function(aText) {
      this.bridgeStatusDisplayfield.setValue(aText);
    },
    
  
  });
}




// @class DSS.addon.huecontrol
Ext.define('DSS.addon.huecontrol', {

  extend: 'DSS.addon.Framework',

  uses : ['DSS.addon.huecontrol.MainPanel', 'Ext.Component'],

  config: {
    appName: _("Hue Control"),
    appVersion: '0.1',
    appId: 'huecontrol',
    appIcon : 'images/app_icon.png',
    appLang : 'en_US'
  },

  /**
   * The content generation function
   * @return {Object} The instantiated content object
   */
  getContent : function() {
    var me=this;
    // get zones
    var zones = [];
    Ext.Array.each(dss.staticDataModel.getZoneArray(),function(entry) {
      zones.push({
        ZoneID : entry.ZoneID,
        name : unescape(entry.name)
      })
    });
    
    var oConfig={
      defaultSortCol:'light',
      storeConfig:{
        storeId : 'hleStore',
        fields : [
          {name:'lightID', sortType:'asUCText'}, 
          {name:'name', sortType:'asUCText'}, 
          {name:'zoneID', sortType:'asUCText'},
		   {name:'dontCare0', type:'boolean'},
		   {name:'dontCare1', type:'boolean'},
		   {name:'dontCare2', type:'boolean'},
		   {name:'dontCare3', type:'boolean'},
		   {name:'dontCare4', type:'boolean'},
		   {name:'dontCare10', type:'boolean'},
		   {name:'dontCare11', type:'boolean'},
		   {name:'dontCare12', type:'boolean'},
		   {name:'dontCare13', type:'boolean'},
		   {name:'dontCare14', type:'boolean'},
		   {name:'dontCare20', type:'boolean'},
		   {name:'dontCare21', type:'boolean'},
		   {name:'dontCare22', type:'boolean'},
		   {name:'dontCare23', type:'boolean'},
		   {name:'dontCare24', type:'boolean'},
		   {name:'dontCare30', type:'boolean'},
		   {name:'dontCare31', type:'boolean'},
		   {name:'dontCare32', type:'boolean'},
		   {name:'dontCare33', type:'boolean'},
		   {name:'dontCare34', type:'boolean'},
		   {name:'dontCare40', type:'boolean'},
		   {name:'dontCare41', type:'boolean'},
		   {name:'dontCare42', type:'boolean'},
		   {name:'dontCare43', type:'boolean'},
		   {name:'dontCare44', type:'boolean'},
                   {name:'dontCare65', type:'boolean'},
                   {name:'dontCare71', type:'boolean'},
                   {name:'dontCare72', type:'boolean'},
                   {name:'dontCare74', type:'boolean'},
                   {name:'dontCare76', type:'boolean'},
                   {name:'dontCare73', type:'boolean'},
        ]
      },
      columns:[
        {
          header : _('Hue Light No'),
          dataIndex : 'lightID',
          xtype: 'textcolumn',
          width : 100
        },
        {
          header : _('Zone'),
          dataIndex : 'zoneID',
          xtype: 'textcolumn',
          width : 200,
          customColumnRenderer : function(val, metadata, record, rowIdx, colIdx, store, view) {
            var dispVal = val; // default to untranslated
            Ext.Array.each(zones,function(entry) {
              if (entry.ZoneID==val) {
                dispVal = entry.name;
                return false;
              }
            });
            return dispVal;
          }
        },
        {
          header : _('Name'),
          dataIndex : 'name',
          xtype: 'textcolumn',
          maintainFlex : true,
          flex : 5,
        },
      ],
      
      buttonText: {
        editButton: _('edit hue light zone and area assignments, define scenes')
      },
      
      customButtons: [
        {
          text: _('configure hue bridge URL (IP address)'),
          iconCls: 'icon-test-action',
          handler: function(grid) {              
            grid.fireEvent('configBridge',grid,grid.curSelectedEvent);
          }
        }
      ],
      

      editorConfig:{
        title : _('Edit hue light parameters'),
        staticControls : [
          {
            type : 'custom',
            id : 'lightID',
            control : Ext.create('Ext.form.field.Display', {
              name : 'lightID',
              fieldLabel: _('Hue Light No'),
            }),
          },
          {
            type : 'text',
            label : _('Name'),
            id : 'name'
          },
          {
            type : 'custom',
            id : 'zoneID',
            control : Ext.create('Ext.form.ComboBox', {
              name : 'zoneID',
              fieldLabel: _('Zone'),
              store: Ext.create('Ext.data.Store', {
                fields: ['ZoneID', 'name'],
                data : zones
              }),
              queryMode: 'local',
              displayField: 'name',
              valueField: 'ZoneID',
            }),            
          }, 
        ],
        tabControls : [
          {
            tabLabel : _('Scene settings'),
            tabDescription : _('Call light settings from scenes or save current light setting to a scene'),
            content : [
              { type : 'custom', control: lightSceneButtons(0,this),id:'dontCare0' },
              { type : 'custom', control: lightSceneButtons(1,this),id:'dontCare1' },
              { type : 'custom', control: lightSceneButtons(2,this),id:'dontCare2' },
              { type : 'custom', control: lightSceneButtons(3,this),id:'dontCare3' },
              { type : 'custom', control: lightSceneButtons(4,this),id:'dontCare4' }
            ]
          },
          {
            tabLabel : _('Apartment Scenes'),
            tabDescription : _('Call light settings from scenes or save current light setting to a scene'),
            content : [
              { type : 'custom', control: lightSceneButtons(65,this),id:'dontCare65' },
              { type : 'custom', control: lightSceneButtons(71,this),id:'dontCare71' },
              { type : 'custom', control: lightSceneButtons(72,this),id:'dontCare72' },
              { type : 'custom', control: lightSceneButtons(74,this),id:'dontCare74' },
              { type : 'custom', control: lightSceneButtons(76,this),id:'dontCare76' },
              { type : 'custom', control: lightSceneButtons(73,this),id:'dontCare73' }
            ]
          },
          {
            tabLabel : _('Area 1 Scenes'),
            tabDescription : _('Call light settings from scenes or save current light setting to a scene'),
            content : [
              { type : 'custom', control: lightSceneButtons(10,this),id:'dontCare10' },
              { type : 'custom', control: lightSceneButtons(11,this),id:'dontCare11' },
              { type : 'custom', control: lightSceneButtons(12,this),id:'dontCare12' },
              { type : 'custom', control: lightSceneButtons(13,this),id:'dontCare13' },
              { type : 'custom', control: lightSceneButtons(14,this),id:'dontCare14' }
            ]
          },
          {
            tabLabel : _('Area 2 Scenes'),
            tabDescription : _('Call light settings from scenes or save current light setting to a scene'),
            content : [
              { type : 'custom', control: lightSceneButtons(20,this),id:'dontCare20' },
              { type : 'custom', control: lightSceneButtons(21,this),id:'dontCare21' },
              { type : 'custom', control: lightSceneButtons(22,this),id:'dontCare22' },
              { type : 'custom', control: lightSceneButtons(23,this),id:'dontCare23' },
              { type : 'custom', control: lightSceneButtons(24,this),id:'dontCare24' }
            ]
          },
          {
            tabLabel : _('Area 3 Scenes'),
            tabDescription : _('Call light settings from scenes or save current light setting to a scene'),
            content : [
              { type : 'custom', control: lightSceneButtons(30,this),id:'dontCare30' },
              { type : 'custom', control: lightSceneButtons(31,this),id:'dontCare31' },
              { type : 'custom', control: lightSceneButtons(32,this),id:'dontCare32' },
              { type : 'custom', control: lightSceneButtons(33,this),id:'dontCare33' },
              { type : 'custom', control: lightSceneButtons(34,this),id:'dontCare34' }
            ]
          },
          {
            tabLabel : _('Area 4 Scenes'),
            tabDescription : _('Call light settings from scenes or save current light setting to a scene'),
            content : [
              { type : 'custom', control: lightSceneButtons(40,this),id:'dontCare40' },
              { type : 'custom', control: lightSceneButtons(41,this),id:'dontCare41' },
              { type : 'custom', control: lightSceneButtons(42,this),id:'dontCare42' },
              { type : 'custom', control: lightSceneButtons(43,this),id:'dontCare43' },
              { type : 'custom', control: lightSceneButtons(44,this),id:'dontCare44' }
            ]
          }
        ],
      },

      listeners:{

        'loadDataStore':function (grid){
          // load right now
          me.loadParameter(grid);
          // also issue a name update and reload when done
          dss.raiseEvent('plan44-addon-huecontrol-config', {
            action : 'updateNames',
            values : false
          });
          me.waitAndReload(grid);          
        },

        'saveEntry':function (grid,values){
          // build areas
          var areas = [];
          for (var ai=1; ai<=4; ai++) {
            if (values['area'+String(ai)]) {
              areas.push(ai);
            }
          }
          values.areas = Ext.JSON.encode(areas);
        
          dss.raiseEvent('plan44-addon-huecontrol-config', {
            action : 'save',
            values : Ext.JSON.encode(values)
          });
          me.waitAndReload(grid);
        },

        'configBridge':function (grid,entry) {
          // open the bridge config panel
          me.bridgeConfigWindow.show();
          // get current URL setting
          var url = me.getTextProp('bridgeURL');
          me.bridgeConfigWindow.setURLValue(url);
          // show the bridge status
          if (me.bridgeConnected()) {
            me.bridgeConfigWindow.setStateText(_('connected'));          
          }
          else {
            me.bridgeConfigWindow.setStateText(_('no connection, please make sure URL is correct'));                      
          }
          me.bridgeConfigWindow.grid = grid;
        },
        
      }
    };
    
    me.bridgeConfigWindow = Ext.create('DSS.addon.huecontrol.bridgeParamEdit', {});

    me.bridgeConfigWindow.on({
      'save' : function(oValue) {
        me.saveBridgeURL(me.bridgeConfigWindow.grid, oValue);
      }
    });
    
    return Ext.create('DSS.gridPanel', oConfig);
    
  },

  
  bridgeConfigWindow : null,
  
  
  updateBridgeURL : function(aNewURL)
  {
    dss.raiseEvent('plan44-addon-huecontrol-config', {
      action : 'saveBridgeURL',
      values : Ext.JSON.encode({ url:aNewURL })
    });
  },


  getTextProp : function(aPropName) {
    var result = '';
    var sString = dss.ajaxSyncRequest('/json/property/getString', {
      'path' : '/scripts/huecontrol/' + aPropName
    });
    if (sString) {
      var sresult = Ext.JSON.decode(sString);
      if (sresult && sresult.result && sresult.result.value)
        result = sresult.result.value;
    }
    return result;
  },


  bridgeConnected : function() {
    // test if we got a API token now
    var apitoken = this.getTextProp('bridgeApiToken');
    return apitoken.length>0;
  },
  
  
  saveBridgeURL : function(grid,value) {
    var me = this;
    this.updateBridgeURL(value);
    // wait for reload
    this.waitAndReload(grid, function() {
      if (!me.bridgeConnected()) {
        // got not API token, might need pressing link button
        Ext.Msg.show({
          title: _('registering with hue bridge'),
          msg: _('Please press the link button on your hue bridge, then click OK'),
          width: 400,
          buttons: Ext.Msg.OK,
          fn: function() {
            // try again
            me.updateBridgeURL(value);
            me.waitAndReload(grid, function() {
              if (!me.bridgeConnected()) {
                Ext.Msg.show({
                  title: _('hue bridge not connected'),
                  msg: _('Cannot connect to the hue bridge - please verify URL is corrent and save it again'),
                  width: 400,
                  buttons: Ext.Msg.OK,
                  icon: Ext.window.MessageBox.ERROR
                });
              }
              else {
                Ext.Msg.show({
                  title: _('hue bridge connected'),
                  msg: _('Successfully connected to hue bridge'),
                  width: 400,
                  buttons: Ext.Msg.OK,
                  icon: Ext.window.MessageBox.INFO
                });                
              }
            },15000); // timout after 15 seconds
          },
          icon: Ext.window.MessageBox.WARNING
        });      
        
      }
    },15000); // timout after 15 seconds
  },
  

  waitAndReload : function(grid,callBack,timeoutMS) {
    if (typeof(timeoutMS)==='undefined') timeoutMS = 60*1000; // 60 seconds by default
    var me = this;
    var iSubscriptID = Math.floor(Math.random() * 100000000000);
    dss.ajaxSyncRequest('/json/event/subscribe', {
      subscriptionID : iSubscriptID,
      name : 'plan44-addon-huecontrol-config-saved'
    });
    dss.ajaxAssyncRequestWithoutAnswer('/json/event/get', {
      subscriptionID : iSubscriptID,
      timeout : timeoutMS
    }, function() {
      // reload parameters
      me.loadParameter(grid);
      // also invoke callback, if any
      if (typeof(callBack)!=='undefined') callBack(grid);
      // unsubscribe from event
      dss.ajaxAssyncRequestWithoutAnswer('/json/event/unsubscribe', {
        subscriptionID : iSubscriptID,
        name : 'plan44-addon-huecontrol-config-saved'
      });
    });
  },

  loadParameter : function(grid) {
    // init the cache
    dss.staticDataModel.initHLECache();
    // save reference to the grid
    this.theGrid = grid;
    // load the lights
    var sString = dss.ajaxSyncRequest('/json/property/query', {
      'query' : '/scripts/huecontrol/lights/*(*)'
    });
    var result = Ext.JSON.decode(sString);
    var oResultObject=dss.parsePropertyQueryAnswer(result,[
      {name:'lightID',type:'simple'},
      {name:'name',type:'simple'},
      {name:'zoneID',type:'simple'},
      {name:'areas',type:'simple'}
    ]);
    // for each light, get scenes
    Ext.Array.each(oResultObject,function(entry) {
      // get scenes
      var sString = dss.ajaxSyncRequest('/json/property/query', {
        'query' : '/scripts/huecontrol/lights/light'+String(entry.lightID)+'/*(*)'
      });
      var sresult = Ext.JSON.decode(sString);
      var sResultObject=dss.parsePropertyQueryAnswer(sresult,[
        {name:'sceneID',type:'simple'},
        {name:'name',type:'simple'},
        {name:'state',type:'simple'},
	    {name:'dontCare',type:'simple'}
      ]);
      var scenes = [];
      Ext.Array.each(sResultObject, function(subentry) {
        if(subentry.sceneID) {
        	scenes.push(subentry);
        	entry['dontCare' + subentry.sceneID]= (subentry.dontCare!="0") && (subentry.dontCare!="false");
        }
      });
      entry['scenes'] = scenes;
    });
    
    if (oResultObject!=null) {
      grid.loadParameter(oResultObject);
    }
  },

  /**
   * The help content generation function
   * @return {Object} The instantiated help content object
   */
  getHelp : function() {
    return Ext.create('Ext.Component', {
      loader : {
        url : 'locale/' + dss.staticDataModel.activeLanguage + '/help.html',
        autoLoad : true,
        disableCaching : false
      }
    });
  },
});



Ext.onReady(function() {
  // - create locale
  dss.buildUpLang(['locale/{languageSuffix}/plan44-huecontrol.po']);
  createNeededClasses();
  // - create the addon object
  var huecontrol = Ext.create('DSS.addon.huecontrol');
  // - kick off page rendering
  huecontrol.initPage();
});


